#include "vmm_tbl.h"

/** Constructor for the VMMTbl class */
VMMTbl::VMMTbl(RMM& rmm, int ring, int node)
  : FrontEndBase(rmm.get_rmm_reg_access(), ring, node),
    pVMMConfig(*this),
    pRMM(&rmm)
{
  addr_list = ring * NUMBER_NODES_PER_RING + node;
  fe_addr_map[ring] = vmm_tbl_register_map;
  create_epics_params();
  set_initial_epics_params();
  pRMM->get_rmm_rings().set_topology(ring, node, "vmm-tbl");

  pVMMConfig.configVMM(0, 0);
  pVMMConfig.readADC(0, 0);

  pVMMConfig.configVMM(0, 1);
  pVMMConfig.readADC(0, 1);
}

void VMMTbl::config_FE() {
  puts("config_FE");
}

void VMMTbl::set_initial_epics_params() {
  puts("set_initial_epics_params");
}

asynStatus VMMTbl::create_epics_params() {
  pRMM->createParam(addr_list, "SEL_ANALOG_MONITOR_VMM0", asynParamInt32, &vmmSelectMonitorVMM0_);
  pRMM->createParam(addr_list, "SEL_ANALOG_MONITOR_VMM1", asynParamInt32, &vmmSelectMonitorVMM1_);
  pRMM->createParam(addr_list, "READ_ADC_VMMS", asynParamInt32, &vmmProcessReadMonitorVMMs_);
  pRMM->createParam(addr_list, "ADC_VALUE_VMM0", asynParamFloat64, &vmmMonitorValueVMM0_);
  pRMM->createParam(addr_list, "ADC_VALUE_VMM1", asynParamFloat64, &vmmMonitorValueVMM1_);

  return asynSuccess;
}

asynStatus VMMTbl::writeInt32(asynUser *pasynUser, epicsInt32 value) {
  asynStatus status = asynSuccess;
  int function = pasynUser->reason;
  int addr = 0;
  pRMM->getAddress(pasynUser, &addr);

  if ((value) && (function == vmmProcessReadMonitorVMMs_)) {
    int adc_sensor = 0;
    for (int vmm = 0; vmm <= 1; vmm++) {
      pRMM->getIntegerParam(addr, vmmSelectMonitorVMM0_ + vmm, &adc_sensor);
      std::cout << "adc_sensor" << adc_sensor << std::endl;
      pVMMConfig.getHybrid(0).getVMM(vmm).setRegister("sm5_sm0", adc_sensor + 64);
      pVMMConfig.configVMM(0, vmm);
      int adc_read = pVMMConfig.readADC(0, vmm);

      std::cout << "adc_read" << adc_read << std::endl;

      if (adc_sensor == 3) {
        double temperature = (725 - adc_read) / 1.85;
        pRMM->setDoubleParam(addr, vmmMonitorValueVMM0_ + vmm, temperature);
      }
      else {
        pRMM->setDoubleParam(addr, vmmMonitorValueVMM0_ + vmm, adc_read);
      }
    }
  }

  return status;
}

asynStatus VMMTbl::writeFloat64(asynUser *pasynUser, epicsFloat64 value) {
  asynStatus status = asynSuccess;
  int function = pasynUser->reason;
  int addr = 0;
  pRMM->getAddress(pasynUser, &addr);

  return status;
}


extern "C" {
  static void VMMTblConfig(const char *rmm_port_name, int ring, int node) {
    RMM *pRMM;
    pRMM = (RMM *) findAsynPortDriver(rmm_port_name);
    pRMM->frontends[ring][node] = std::make_shared<VMMTbl>(*pRMM, ring, node);
  }

  static const iocshArg configArg0 = { "RMM Port name", iocshArgString};
  static const iocshArg configArg1 = { "Ring", iocshArgInt};
  static const iocshArg configArg2 = { "Node", iocshArgInt};


  static const iocshArg * const configArgs[] = {&configArg0,
                                                &configArg1,
                                                &configArg2};

  static const iocshFuncDef configFuncDef = {"VMMTblConfig", 3, configArgs};

  static void configCallFunc(const iocshArgBuf *args) {
    VMMTblConfig(args[0].sval, args[1].ival, args[2].ival);
  }

  static void VMMTblRegister(void) {
    iocshRegister(&configFuncDef, configCallFunc);
  }

  epicsExportRegistrar(VMMTblRegister);
} // end extern "C"